
// -*- mode: c++; c-basic-offset:4 -*-

#ifdef __GNUG__
#pragma interface
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
#include <io.h>			// stat for win32? 09/05/02 jhrg
#else
#include <unistd.h>
#endif

#include <string>
#include <vector>

#include "Error.h"
#include "InternalErr.h"

const int CACHE_TABLE_SIZE = 1024;

using namespace std;

/** Implements a single-user, client-side, HTTP 1.1 compliant cache.

    TODO: Create the cache directories, Implement locking, enable/disable
    toggle, validation test, Add a response to the cache, check the cache for
    a hit.
*/
class HTTPCache {
private:
    string d_cache_root;
    string d_cache_index;
    FILE *d_locked_open_file;

#if 0
    // Final states have negative value
    enum CacheState {
	CL_ERROR = -3,
	CL_NO_DATA = -2,
	CL_GOT_DATA = -1,
	CL_BEGIN = 0,
	CL_NEED_BODY,
	CL_NEED_OPEN_FILE,
	CL_NEED_CONTENT
    };

    /* This is the context structure for the this module */
    struct CacheInfo {
	CacheState state;	/* Current state of the connection */
	char *local;		/* Local representation of file name */
	struct stat stat_info;	/* Contains actual file chosen */
#if 0
	HTNet *net;
	HTTimer *timer;		// Timer callback from libwww; remove?
#endif
    };
#endif

    struct CacheEntry {
	// Location
	string url;
	string cachename;

	// GC parameters
	string etag;
	time_t lm;		// Last modified
	time_t expires;
	long size;		// Size of cached entity body
	bool range;		// Is this the full part or a subpart?
	int hash;
	int hits;		// Hit counts
	time_t freshness_lifetime;
	time_t response_time;
	time_t corrected_initial_age;
	bool must_revalidate;
#if 0
	HTRequest *lock;
#endif
    };

    // Typedefs for CacheTable. A CacheTable is a vector of vectors of
    // CacheEntries. The outer vector is accessed using the hash value.
    // Entries with matching hashes occupy successive positions in the inner
    // vector (that's how hash collisions are resolved).
    typedef vector<CacheEntry *> CachePointers;
    typedef CachePointers::iterator CachePointersIter;

    typedef CachePointers *CacheTable[CACHE_TABLE_SIZE];

    CacheTable d_cache_table;

    // Suppress these methods.
    void clone(const HTTPCache &cache) {}

    HTTPCache(const HTTPCache &cache) { 
	clone(cache); 
    }

    HTTPCache &operator=(const HTTPCache &rhs) { 
	if (this != &rhs)
	    clone(rhs);
	return *this;
    }
	
    CacheEntry *cache_index_parse_line(char *line) throw(InternalErr);
    void add_entry_to_cache_table(CacheEntry *e);
    CacheEntry *get_entry_from_cache_table(int hash, const string &url);
    void create_cache_root(const string &cache_root) throw(Error);

    friend class HTTPCacheTest;

public:
    HTTPCache(string cache_root);
    virtual ~HTTPCache();

    bool cache_index_delete();
    bool cache_index_read();
    void cache_index_write() throw(Error);

    void set_cache_root(const string &root = "") throw(Error);
    string get_cache_root();
    bool get_single_user_lock(bool force = false);
};

// $Log: HTTPCache.h,v $
// Revision 1.1.2.2  2002/09/17 00:16:45  jimg
// Added methods to lock the cache and find/set its root directory.
//
// Revision 1.1.2.1  2002/09/12 22:38:55  jimg
// Preliminary version. Added at this time because the Makefile includes it.
//
