
// -*- mode: c++; c-basic-offset:4 -*-

#include <stdio.h>		// for create_cache_root_test
#include <unistd.h>		// ibid [sic]

#include <cppunit/TextTestRunner.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>

#if 0
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <string>
#include <strstream.h>

#include <Regex.h>
#endif

#include "HTTPCache.h"

using namespace CppUnit;

class HTTPCacheTest : public TestFixture {
private:
    HTTPCache *hc;
    HTTPCache *hc_2;
    char *index_file_line;
    char *test_url;
    int hash_value;

protected:

public:
    HTTPCacheTest() {}
    ~HTTPCacheTest() {}
    
    void setUp () {
	hc = new HTTPCache("cache-testsuite/dods_cache/");
	hc_2 = new HTTPCache("cache-testsuite/dods_cache/");
	hash_value = 391;	
	test_url = "http://dcz.dods.org/dods-3.2/nph-dods/data/nc/fnoc1.nc.dds";
	index_file_line = "http://dcz.dods.org/dods-3.2/nph-dods/data/nc/fnoc1.nc.dds /usr/local/tmp/.dods_cache/391/file1qr09a @w3c@ 930090223 -1 197 0 391 1 172800 1030124259 1 0";
    }

    void tearDown() {}

    CPPUNIT_TEST_SUITE(HTTPCacheTest);

    CPPUNIT_TEST(constructor_test);
    CPPUNIT_TEST(cache_index_read_test);
    CPPUNIT_TEST(cache_index_parse_line_test);
    CPPUNIT_TEST(get_entry_from_cache_table_test);
    CPPUNIT_TEST(cache_index_write_test);
    CPPUNIT_TEST(create_cache_root_test);
    CPPUNIT_TEST(set_cache_root_test);
    CPPUNIT_TEST(get_single_user_lock_test);

    CPPUNIT_TEST_SUITE_END();

    void constructor_test() {
	CPPUNIT_ASSERT(hc->d_cache_index=="cache-testsuite/dods_cache/.index");
	CPPUNIT_ASSERT(hc->d_cache_root == "cache-testsuite/dods_cache/");
    }

    void cache_index_read_test() {
	hc->cache_index_read();

	HTTPCache::CacheEntry *e = hc->get_entry_from_cache_table(hash_value, 
								  test_url);
	CPPUNIT_ASSERT(e->url == test_url);
    }

    void cache_index_parse_line_test() {
	HTTPCache::CacheEntry *e = hc->cache_index_parse_line(index_file_line);

	CPPUNIT_ASSERT(e->url == test_url);
	CPPUNIT_ASSERT(e->cachename 
		       == "/usr/local/tmp/.dods_cache/391/file1qr09a");
	CPPUNIT_ASSERT(e->etag == "");
	CPPUNIT_ASSERT(e->lm == 930090223);
	// Skip ahead ...
	CPPUNIT_ASSERT(e->must_revalidate == false);
    }

    // This will also test the add_entry_to_cache_table() method.
    void get_entry_from_cache_table_test() {
	HTTPCache::CacheEntry *test_ce 
	    = hc_2->cache_index_parse_line(index_file_line);

	// Test adding an entry and getting it back.
	hc_2->add_entry_to_cache_table(test_ce);

	HTTPCache::CacheEntry *e 
	    = hc_2->get_entry_from_cache_table(hash_value, test_url);
	CPPUNIT_ASSERT(e->url == test_url);

	// Now test what happens when two entries collide.
	HTTPCache::CacheEntry *test_ce2 
	    = hc_2->cache_index_parse_line(index_file_line);
	// Change the url so we can tell the difference (the hash is the same)
	test_ce2->url = "http://new.url.same.hash/test/collisions.gif";

	hc_2->add_entry_to_cache_table(test_ce2);

	HTTPCache::CacheEntry *g 
	    = hc_2->get_entry_from_cache_table(hash_value, 
			       "http://new.url.same.hash/test/collisions.gif");
	CPPUNIT_ASSERT(g->url
		       == "http://new.url.same.hash/test/collisions.gif");
    }

    void cache_index_write_test() {
	HTTPCache *hc_3 = new HTTPCache("cache-testsuite/dods_cache/");
	hc_3->add_entry_to_cache_table(hc_2->cache_index_parse_line(index_file_line));

	hc_3->d_cache_index = hc_2->d_cache_root + "test_index";
	hc_3->cache_index_write();

	HTTPCache *hc_4 = new HTTPCache("cache-testsuite/dods_cache/");
	hc_4->d_cache_index = hc_3->d_cache_root + "test_index";
	hc_4->cache_index_read();

	HTTPCache::CacheEntry *e 
	    = hc_4->get_entry_from_cache_table(hash_value, test_url);
	CPPUNIT_ASSERT(e);
	CPPUNIT_ASSERT(e->url == test_url);
    }

    void create_cache_root_test() {
	hc->create_cache_root("/tmp/silly/");
	CPPUNIT_ASSERT(access("/tmp/silly/", F_OK) == 0);
	remove("/tmp/silly");
	try {
	    hc->create_cache_root("/root/very_silly/");
	    access("/root/very_silly/", F_OK);
	    remove("/root/very_silly/");
	    CPPUNIT_ASSERT(false && "Should not be able to do this...");
	}
	catch (Error &e) {
	    CPPUNIT_ASSERT("This is where we want to be");
	    CPPUNIT_ASSERT(access("/root/very_silly/", F_OK) != 0);
	}
    }

    void set_cache_root_test() {
	putenv("WWW_CACHE=/home/jimg");
	hc->set_cache_root();
	CPPUNIT_ASSERT(hc->d_cache_root == "/home/jimg/w3c-cache/");
	remove("/home/jimg/w3c-cache/");

	hc->set_cache_root("/home/jimg/test_cache");
	CPPUNIT_ASSERT(hc->d_cache_root == "/home/jimg/test_cache/");
	remove("/home/jimg/test_cache/");

	try {
	    hc->set_cache_root("/root/never/ever/");
	    CPPUNIT_ASSERT(false && "Created cache in root's dir. Bad.");
	}
	catch (Error &e) {
	    CPPUNIT_ASSERT("OK, caught an error");
	}

	remove("/root/never/ever/");
    }

    void get_single_user_lock_test() {
	hc->set_cache_root("/tmp/dods_test_cache");

	CPPUNIT_ASSERT(hc->get_single_user_lock());
	CPPUNIT_ASSERT(access("/tmp/dods_test_cache/.lock", F_OK) == 0);

	// Second time should fail
	CPPUNIT_ASSERT(!hc->get_single_user_lock());

	// Should fail, too
	CPPUNIT_ASSERT(!hc_2->get_single_user_lock());
	// Should succeed (force == true)
	CPPUNIT_ASSERT(hc_2->get_single_user_lock(true));

	// Get rid on the lock
      	remove("/tmp/dods_test_cache/.lock");
    }
	
};

CPPUNIT_TEST_SUITE_REGISTRATION(HTTPCacheTest);

int 
main( int argc, char* argv[] )
{
    CppUnit::TextTestRunner runner;
    runner.addTest( CppUnit::TestFactoryRegistry::getRegistry().makeTest() );

    runner.run();

    return 0;
}

// $Log: HTTPCacheTest.cc,v $
// Revision 1.1.2.1  2002/09/17 00:19:29  jimg
// New tests for cache root and lock code.
//
// Revision 1.1.2.5  2002/06/19 06:05:36  jimg
// Built a single file version of the cgi_util tests. This should be easy to
// clone for other test suites.
//
