
// (c) COPYRIGHT URI/MIT 1994-2002
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>
//      dan             Dan Holloway <dholloway@gso.uri.edu>
//      reza            Reza Nekovei <reza@intcomm.net>

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

static char rcsid[] not_used =
    { "$Id: HTTPConnect.cc,v 1.1.2.8 2002/09/14 03:40:32 jimg Exp $" };

#include <stdio.h>
#ifndef WIN32
#include <unistd.h>
#else
#include <io.h>
#endif

#if 0
#include <strstream>
#endif
#include <sstream>

#include "debug.h"
#include "Regex.h"
#include "HTTPConnect.h"
#include "RCReader.h"

using std::cerr;
using std::endl;
using std::iterator;
using std::string;
using std::istrstream;

#ifdef WIN32
using std::vector<string>;
#else
using std::vector;
#endif


// These two global variables are not MT-Safe, but I'm leaving them as is
// because they are used only for debugging (set them in a debugger like gdb
// or ddd). They are not static because I *believe* that debuggers cannot
// access static variables. 08/07/02 jhrg

// This is a convenience used when running this software under a debugger.
// Set the global 'keep_temps' to 1 to keep all the temporary files used to
// store libwww retrieved documents. 6/17/2002 jhrg
int keep_temps = 0;

// Set this to 1 to turn on libcurl's verbose mode (for debugging).
int www_trace = 0;

// Use the stream parameter to pass in 'this'; read headers, line by line,
// from ptr. The fourth param is really supposed to be a FILE *, but libcurl
// just holds the pointer and passes it to this function without using it
// itself. I use that to pass in a pointer to the HTTPConnect that initiated
// the HTTP request so that there's some place to dump the headers.
//
// To support caching we need all of the response's headers. I've added the
// capacity to record them in the HTTPConnect. 09/13/02 jhrg
size_t 
parse_headers(void *ptr, size_t size, size_t nmemb, void *http_connect)
{
    DBG2(cerr << "Inside the header parser." << endl);
    HTTPConnect *http = static_cast<HTTPConnect *>(http_connect);

#if 0
    istrstream line(static_cast<char *>(ptr), size * nmemb);
#endif
    string complete_line(static_cast<char *>(ptr), size * nmemb - 1);
    // Store all non-empty headers that are not HTTP status codes
    if (complete_line != "" && complete_line.find("HTTP") == string::npos) {
	DBG(cerr << "Header line: " << complete_line << endl);
	http->d_headers.push_back(complete_line);
    }

    istringstream line(complete_line);
    string header;
    line >> header;
    downcase(header);
    if (header == "content-description:") {
	string value; 
	line >> value;
	downcase(value);
	DBG(cout << header << ": " << value << endl);
	http->d_type = get_type(value);
    }
    else if (header == "xdods-server:") {
	string value; 
	line >> value;
	downcase(value);
	DBG(cout << header << ": " << value << endl);
	http->d_server = value;
    }
    else if (http->d_server == "dods/0.0" && header == "server:") {
	string value; 
	line >> value;
	downcase(value);
	DBG(cout << header << ": " << value << endl);
	http->d_server = value;
    }

    return size * nmemb;
}

/** Initialize libcurl. Create a libcurl handle that can be used for all of
    the HTTP requests made through this instance. */ 
void 
HTTPConnect::www_lib_init() throw(Error, InternalErr)
{
    d_curl = curl_easy_init();
    if (!d_curl)
	throw InternalErr(__FILE__, __LINE__, "Could not initialize libcurl.");

    // Now set options that will remain constant for the duration of this
    // CURL object.

    // We must set the proxy host (it's really a host, if a URL is given the
    // protocol part is ignored by libcurl). If the port is not given with the
    // proxy server host, then we figure it out using the separate proxy
    // protocol information from the configuration file. 
    if (d_rcr->get_proxy_server_host_url() != "") {
	string proxy = d_rcr->get_proxy_server_host_url();
	curl_easy_setopt(d_curl, CURLOPT_PROXY, proxy.c_str());
	// If the port number is not part of the proxy server host, then use
	// the protocol to divine a default port number. Support http, https
	// and ftp for now. If protocol is not one of those, throw and error.
	Regex find_port("^.*:[0-9]+$");
	int index=0, matchlen;
	if (find_port.search(proxy.c_str(), proxy.length(), matchlen, index)
	    == -1) {
	    string protocol = d_rcr->get_proxy_server_protocol();
	    if (protocol == "http")
		curl_easy_setopt(d_curl, CURLOPT_PROXYPORT, (long)80);
	    else if (protocol == "https")
		curl_easy_setopt(d_curl, CURLOPT_PROXYPORT, (long)443);
	    else if (protocol == "ftp")
		curl_easy_setopt(d_curl, CURLOPT_PROXYPORT, (long)21);
	    else 
		throw Error("Proxies for the protocol '" + protocol 
			    + "' are not supported.");
	}
    }

    curl_easy_setopt(d_curl, CURLOPT_ERRORBUFFER, d_error_buffer);
    curl_easy_setopt(d_curl, CURLOPT_FAILONERROR, 1);

    curl_easy_setopt(d_curl, CURLOPT_NOPROGRESS, 1);
    curl_easy_setopt(d_curl, CURLOPT_MUTE, 1);

    curl_easy_setopt(d_curl, CURLOPT_HEADERFUNCTION, parse_headers);
    // By passing `this' to the writeheader function we can load values read
    // from the headers into this object.
    curl_easy_setopt(d_curl, CURLOPT_WRITEHEADER, this);

    if (www_trace)
	curl_easy_setopt(d_curl, CURLOPT_VERBOSE, 1);
}

// Use the URL designated when the Connect object was created as the
// `base' URL so that the formal parameter to this mfunc can be relative.

/* Assume that the object's \_OUTPUT stream has been set properly. Error
   signals something's wrong. */
void
HTTPConnect::read_url(const string &url, FILE *stream) throw(Error)
{
    curl_easy_setopt(d_curl, CURLOPT_URL, url.c_str());
    curl_easy_setopt(d_curl, CURLOPT_FILE, stream);

    bool temporary_proxy = false;
    if ((temporary_proxy = url_uses_proxy_for(url))) {
	// Set the new proxy.
	curl_easy_setopt(d_curl, CURLOPT_PROXY,
			 d_rcr->get_proxy_for_proxy_host_url().c_str());
    }
    else if ((temporary_proxy = url_uses_no_proxy_for(url))) {
	// Turn off the proxy for this URL.
	curl_easy_setopt(d_curl, CURLOPT_PROXY, 0);	
    }

    CURLcode res = curl_easy_perform(d_curl);

    // Reset the proxy?
    if (temporary_proxy && d_rcr->get_proxy_server_host_url() != "")
	curl_easy_setopt(d_curl, CURLOPT_PROXY,
			  d_rcr->get_proxy_server_host_url().c_str());
	
    if (res != 0)
	throw Error(d_error_buffer);
}

/** If the .dodsrc file gives a value for PROXY_FOR, return true if the
    current URL matches the regular expression. */
bool
HTTPConnect::url_uses_proxy_for(const string &url) throw()
{
    if (d_rcr->is_proxy_for_used()) {
	Regex host_regex(d_rcr->get_proxy_for_regexp().c_str());
	int index=0, matchlen;
	return host_regex.search(url.c_str(), url.size(), matchlen, index)
	    != -1;
    }
    
    return false;	
}

/** If the NO_PROXY option is used in the dodsrc file, does this URL match
    the no proxy URL regex? */
bool
HTTPConnect::url_uses_no_proxy_for(const string &url) throw()
{
    return d_rcr->is_no_proxy_for_used()
	&& url.find(d_rcr->get_no_proxy_for_host()) != string::npos;
}

// Public methods.

/** Build a virtual connection to a remote data source that will be
    accessed using HTTP. 

    @name The URL to the remote data source.
    @param rcr A pointer to the RCReader object which holds configuration
    file information to be used by this virtual connection. */
HTTPConnect::HTTPConnect(RCReader *rcr) throw(Error, InternalErr)
    : d_username(""), d_password(""), d_is_response_present(false),
      d_type(unknown_type), d_server("dods/0.0"), d_output(0)
{
    d_accept_deflate = rcr->get_deflate();
    d_rcr = rcr;

    www_lib_init();		// This may throw either Error or InternalErr
}

HTTPConnect::~HTTPConnect()
{
    DBG2(cerr << "Entering the HTTPConnect dtor" << endl);

#if defined(WIN32) || defined(TEST_WIN32_TEMPS)
    //  Get rid of any intermediate files
    vector < string >::const_iterator i;
    for (i = d_tfname.begin(); i != d_tfname.end(); i++)
	remove((*i).c_str());
#endif

    curl_easy_cleanup(d_curl);

    close_output();

    DBG2(cerr << "Leaving the HTTPConnect dtor" << endl);
}

/** Dereference a URL. This method dereferences a URL and stores the result
    (i.e., if formulates an HTTP request and processes the HTTP server's
    response). After this method is successfully called, the value of
    <c>is_response_present()</c> will be true and the methods
    <c>output()</c>, <c>get_response_headers()</c>, <c>type()</c> and
    <c>server_version()</c> may be called.

    @param url The URL to dereference.
    @exception Error The URL could not be dereferenced.
    @exception InternalErr A temporary file to hold the response could not be
    opened. */
void
HTTPConnect::fetch_url(const string &url) throw(Error, InternalErr)
{
    // Clear/Reset values from previous requests.
    d_type = unknown_type;
    d_headers.clear();
    d_is_response_present = false;

    // get_tempfile_template uses new, must call delete
    char *dods_temp = get_tempfile_template("dodsXXXXXX");

    // Open truncated for update. NB: mkstemp() returns a file descriptor.
#if defined(WIN32) || defined(TEST_WIN32_TEMPS)
    FILE *stream = fopen(_mktemp(dods_temp), "wb");
    if (stream)
	d_tfname.push_back(string(dods_temp));
#else
    FILE *stream = fdopen(mkstemp(dods_temp), "wb");
#endif

    if (!stream)
	throw InternalErr("While dereferencing the url to the dataset, I\n\
 could not open a temporary file for the data values.");

    try {
	read_url(url, stream);	// Throws Error.
    }
    catch(...) {
	delete dods_temp;
	throw;
    }

    // Workaround for Linux 2.2 (only?). Using fseek() did not work for URLs
    // with between 61 and 64 characters in them. I have no idea why, but it
    // appeared in ddd as if the eof was not being reported correctly. Since
    // dumping all the data to an intermediate file is sort of a hack, I
    // decided to close and then reopen the file to see if that would cure
    // the problem. It seems to. The real solution is to pass the socket back
    // via _output. However, this means figuring that out and getting
    // multi-part MIME docs working in DODS. 8/2/2000 jhrg
    fclose(stream);
    stream = fopen(dods_temp, "rb");
    if (!keep_temps)
	unlink(dods_temp);	// When _OUTPUT is closed file is deleted
    else {
	// It's OK to call cerr here since keep_temps is true only when we're
        // debugging. Normally this code will never be called. 09/13/02 jhrg
	cerr << "Temporary file for Data document: " << dods_temp << endl;
    }

    delete dods_temp;

    d_output = stream;
    d_is_response_present = true;
}

/** True if fetch_url() has been called and a response has been received. */
bool
HTTPConnect::is_response_present()
{
    return d_is_response_present;
}

/** Get the HTTP response headers. 
    @return A vector of strings, one string for each header. 
    @exeception IntenalErr The <c>is_response_present</c> property is false. */
vector<string>
HTTPConnect::get_response_headers() throw(InternalErr)
{
    if (!d_is_response_present)
	throw InternalErr(__FILE__, __LINE__, "Caller tried to access invalid response information");

    return d_headers;
}

/** Returns a file pointer which can be used to read the data
    fetched from a URL.

    Note that occasionally this may be directed to #stdout#.  If this
    is the case, users should avoid closing it.

    @memo Access the information contained in this Connect instance.
    @see Connect::fetch_url
    @return A #(FILE *)# indicating a file containing the data
    received from a dereferenced URL.  
    @exeception IntenalErr The <c>is_response_present</c> property is false. */
FILE *
HTTPConnect::output() throw(InternalErr)
{
    if (!d_is_response_present)
	throw InternalErr(__FILE__, __LINE__, "Caller tried to access response invalid response information");

    return d_output;
}

/** Close the output stream of the Connect object. This closes the FILE
    pointer returned by #output()#. In addition, it also deletes the internal
    XDR stream object, although users should not have to know about
    that\ldots Note that calling this method resets (clears) the <i>response
    valid</i> property.

    @memo Close the object's output stream if it is not NULL or STDOUT. */
void 
HTTPConnect::close_output() throw(InternalErr)
{
    d_is_response_present = false;

    if (d_output) {
	if (ferror(d_output))
	    throw InternalErr(__FILE__, __LINE__,
			      "Error detected in the data stream.");
	fclose(d_output);

	d_output = 0;
    }
}

/** During the parse of the message headers returned from the dereferenced
    URL, the object type is set. Use this function to read that type
    information. This will be valid {\it before} the return object is
    completely parsed so it can be used to decide which parser to call to
    read the data remaining in the input stream.

    The object types are Data, DAS, DDS, Error, and undefined.

    @memo What type is the most recent object sent from the server?

    @return The type of the object.
    @see ObjectType 
    @exeception IntenalErr The <c>is_response_present</c> property is false. */
ObjectType 
HTTPConnect::type() throw(InternalErr)
{
    if (!d_is_response_present)
	throw InternalErr(__FILE__, __LINE__, "Caller tried to access response invalid response information");

    return d_type;
}

/** Returns a string containing the version of DODS used by the server. 
    @exeception IntenalErr The <c>is_response_present</c> property is false. */
string 
HTTPConnect::server_version() throw(InternalErr)
{
    if (!d_is_response_present)
	throw InternalErr(__FILE__, __LINE__, "Caller tried to access response invalid response information");

    return d_server;
}

/** Set the credentials for responding to challenges while dereferencing
    URLs. Alternatively, these can be embedded in the URL. This method
    provide a way for clients of HTTPConnect to get credentials from users
    (say using a popup dialog) and not have to hack the URL to pass that
    information to libcurl. 
	
    @param u The username.
    @param p The password. 
    @see extract_auth_info() */
void 
HTTPConnect::set_credentials(string u, string p) throw(InternalErr)
{
    // Store the credentials locally.
    d_username = u;
    d_password = p;

    d_upstring = u + ":" + p;

    if (curl_easy_setopt(d_curl, CURLOPT_USERPWD, d_upstring.c_str()) != 0)
	throw InternalErr(__FILE__, __LINE__, 
	      "Could not set the username and password string using libcurl.");
}

// $Log: HTTPConnect.cc,v $
// Revision 1.1.2.8  2002/09/14 03:40:32  jimg
// Added get_response_headers() method. This method provides a way to access the
// headers included in a response, specifically so that they can be cached.
// Added is_response_present() which returns true when fetch_url() has been
// called and a response is present. Also added tests which cause the four
// methods that access information from the response (output, et c.) to throw an
// InternalErr if they are called and is_response_present() would return false.
//
// Revision 1.1.2.7  2002/08/22 21:23:23  jimg
// Fixes for the Win32 Build made at ESRI by Vlad Plenchoy and myslef.
//
// Revision 1.1.2.6  2002/08/08 06:54:57  jimg
// Changes for thread-safety. In many cases I found ugly places at the
// tops of files while looking for globals, et c., and I fixed them up
// (hopefully making them easier to read, ...). Only the files RCReader.cc
// and usage.cc actually use pthreads synchronization functions. In other
// cases I removed static objects where they were used for supposed
// improvements in efficiency which had never actually been verifiied (and
// which looked dubious).
//
// Revision 1.1.2.5  2002/08/06 22:09:49  jimg
// Removed old code about cache control.
// Fixed set_credentials() so that it is MT-safe.
//
// Revision 1.1.2.4  2002/07/06 20:57:08  jimg
// *** empty log message ***
//
// Revision 1.1.2.3  2002/07/06 20:08:08  jimg
// Added support for proxies. Fixed the deflate stuff so that it matches the new
// `sense' of the _dods_deflate field. Compression still does not work since
// libcurl doesn't handle it yet.
//
// Revision 1.1.2.2  2002/06/20 03:18:48  jimg
// Fixes and modifications to the Connect and HTTPConnect classes. Neither
// of these two classes is complete, but they should compile and their
// basic functions should work.
//
// Revision 1.1.2.1  2002/06/18 21:57:24  jimg
// Created. This is a DODS interface to a HTTP library. This version uses
// libwww. A lot of the methods here should be removed...
//
